/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { clearText, haveValue, typeText } from '../../support/cypress.util';

export class OrganisationseinheitenSignaturE2EComponent {
  private readonly organisationsEinheitName: string = 'organisations-form-container-headline';
  private readonly signatureText: string = 'signature-textarea';
  private readonly saveSignatureButton: string = 'save-button';

  public getOrganisationsEinheitName(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.organisationsEinheitName);
  }

  public getSignatureText(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.signatureText);
  }

  public setSignature(signatur: string): void {
    this.clearSignature();
    typeText(this.getSignatureText(), signatur);
  }

  public clearSignature(): void {
    clearText(this.getSignatureText());
  }

  public getSaveButton(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.saveSignatureButton);
  }

  public saveSignature(): void {
    this.getSaveButton().click();
  }

  public hasSignature(compare: string): void {
    haveValue(this.getSignatureText(), compare);
  }

  public hasScrollbar(): void {
    this.getSignatureText().then((textarea) => {
      const scrollHeight = textarea[0].scrollHeight;
      const clientHeight = textarea[0].clientHeight;

      expect(scrollHeight).to.be.greaterThan(clientHeight);
    });
  }
}
