/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AdminSettingsModule } from '@admin-client/admin-settings';
import { ApiRootModule } from '@alfa-client/api-root-shared';
import { BuildInfoComponent } from '@alfa-client/common';
import { EnvironmentModule } from '@alfa-client/environment-shared';
import { TechSharedModule } from '@alfa-client/tech-shared';
import { CommonModule, registerLocaleData } from '@angular/common';
import { HTTP_INTERCEPTORS, provideHttpClient, withInterceptorsFromDi } from '@angular/common/http';
import localeDe from '@angular/common/locales/de';
import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { RouterModule } from '@angular/router';
import { EffectsModule } from '@ngrx/effects';
import { StoreRouterConnectingModule } from '@ngrx/router-store';
import { StoreModule } from '@ngrx/store';
import { StoreDevtoolsModule } from '@ngrx/store-devtools';
import {
  AdminLogoIconComponent,
  DropdownMenuButtonComponent,
  DropdownMenuComponent,
  LogoutIconComponent,
  MailboxIconComponent,
  NavItemComponent,
  NavbarComponent,
  OrgaUnitIconComponent,
  UsersIconComponent,
} from '@ods/system';
import { OAuthModule } from 'angular-oauth2-oidc';
import { HttpUnauthorizedInterceptor } from 'libs/authentication/src/lib/http-unauthorized.interceptor';
import { UserProfileButtonContainerComponent } from '../common/user-profile-button-container/user-profile.button-container.component';
import { environment } from '../environments/environment';
import { OrganisationsEinheitFormPageComponent } from '../pages/organisationseinheit/organisationseinheit-form-page/organisationseinheit-form-page.component';
import { OrganisationsEinheitPageComponent } from '../pages/organisationseinheit/organisationseinheit-page/organisationseinheit-page.component';
import { PostfachPageComponent } from '../pages/postfach/postfach-page/postfach-page.component';
import { UnavailablePageComponent } from '../pages/unavailable/unavailable-page/unavailable-page.component';
import { UserAddPageComponent } from '../pages/users-roles/user-add-page/user-add-page.component';
import { UserRolesPageComponent } from '../pages/users-roles/user-roles-page/user-roles-page.component';
import { AppComponent } from './app.component';
import { appRoutes } from './app.routes';

registerLocaleData(localeDe);

@NgModule({
  declarations: [
    AppComponent,
    PostfachPageComponent,
    UserRolesPageComponent,
    OrganisationsEinheitPageComponent,
    OrganisationsEinheitFormPageComponent,
    UserProfileButtonContainerComponent,
    UnavailablePageComponent,
    UserAddPageComponent,
  ],
  imports: [
    CommonModule,
    AdminLogoIconComponent,
    DropdownMenuComponent,
    DropdownMenuButtonComponent,
    NavItemComponent,
    NavbarComponent,
    OrgaUnitIconComponent,
    LogoutIconComponent,
    UsersIconComponent,
    MailboxIconComponent,
    RouterModule.forRoot(appRoutes),
    BrowserModule,
    BrowserAnimationsModule,
    ApiRootModule,
    EnvironmentModule,
    environment.production ? [] : StoreDevtoolsModule.instrument({ connectInZone: true }),
    StoreModule.forRoot({}),
    EffectsModule.forRoot(),
    StoreRouterConnectingModule.forRoot(),
    FormsModule,
    ReactiveFormsModule,
    AdminSettingsModule,
    OAuthModule.forRoot({
      resourceServer: {
        sendAccessToken: true,
      },
    }),
    TechSharedModule,
    BuildInfoComponent,
  ],
  providers: [
    {
      provide: HTTP_INTERCEPTORS,
      useClass: HttpUnauthorizedInterceptor,
      multi: true,
    },
    provideHttpClient(withInterceptorsFromDi()),
  ],
  bootstrap: [AppComponent],
})
export class AppModule {}
