/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { singleCold, singleHot } from 'libs/tech-shared/test/marbles';
import { Mock, mock } from 'libs/test-utils/src/lib/mocking';
import { MockComponent } from 'ng-mocks';
import { createPostfachResource } from '../../../../test/postfach/postfach';
import { PostfachResource } from '../postfach.model';
import { PostfachService } from '../postfach.service';
import { PostfachContainerComponent } from './postfach-container.component';
import { PostfachFormComponent } from './postfach-form/postfach-form.component';

describe('PostfachContainerComponent', () => {
  let component: PostfachContainerComponent;
  let fixture: ComponentFixture<PostfachContainerComponent>;

  const postfachService: Mock<PostfachService> = mock(PostfachService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [PostfachContainerComponent, MockComponent(PostfachFormComponent)],
      imports: [ReactiveFormsModule],
      providers: [
        {
          provide: PostfachService,
          useValue: postfachService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PostfachContainerComponent);
    component = fixture.componentInstance;

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call postfach service', () => {
      component.ngOnInit();

      expect(postfachService.get).toHaveBeenCalled();
    });

    it('should set postfach', () => {
      const postfachStateResource: StateResource<PostfachResource> =
        createStateResource(createPostfachResource());
      postfachService.get.mockReturnValue(singleHot(postfachStateResource));

      component.ngOnInit();

      expect(component.postfachStateResource$).toBeObservable(singleCold(postfachStateResource));
    });
  });
});
