/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidLinkRel, BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { VorgangResource, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { Component, Input, OnChanges, SimpleChanges } from '@angular/core';
import { getUrl, hasLink } from '@ngxp/rest';
import { has } from 'lodash-es';
import { Observable, tap } from 'rxjs';

@Component({
  selector: 'alfa-bescheid-in-vorgang-container',
  templateUrl: './bescheid-in-vorgang-container.component.html',
  styles: [],
})
export class BescheidInVorgangContainerComponent implements OnChanges {
  @Input() vorgang: VorgangResource;

  public bescheidStateResource$: Observable<StateResource<BescheidResource>>;
  public bescheidDocumentFile$: Observable<StateResource<BinaryFileResource>>;

  get hasBescheidDraft(): boolean {
    return hasLink(this.vorgang, VorgangWithEingangLinkRel.BESCHEID_DRAFT);
  }

  constructor(private bescheidService: BescheidService) {}

  // TODO Prüfen, warum nicht als Setter möglich. Hatte im ersten Versuch zu Laufzeitfehlern geführt.
  ngOnChanges(changes: SimpleChanges): void {
    if (!has(changes, 'vorgang')) {
      return;
    }

    this.bescheidStateResource$ = this.getBescheid();
    this.bescheidDocumentFile$ = this.bescheidService.getBescheidDocumentFile();
  }

  getBescheid(): Observable<StateResource<BescheidResource>> {
    return this.bescheidService.getBescheidDraft().pipe(
      tap((bescheidResource) => {
        if (hasLink(bescheidResource.resource, BescheidLinkRel.BESCHEID_DOCUMENT)) {
          this.bescheidService.loadBescheidDocumentByUri(
            getUrl(bescheidResource.resource, BescheidLinkRel.BESCHEID_DOCUMENT),
          );
        }
      }),
    );
  }
}
