/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidService } from '@alfa-client/bescheid-shared';
import { CommandResource, tapOnCommandSuccessfullyDone } from '@alfa-client/command-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { createDialogResult } from '@alfa-client/ui';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { Component, Inject } from '@angular/core';
import { Observable } from 'rxjs';

export interface AbschliessenDialogData {
  vorgangWithEingangResource: VorgangWithEingangResource;
}

@Component({
  selector: 'alfa-bescheid-wizard-abschliessen-dialog-container',
  templateUrl: './bescheid-wizard-abschliessen-dialog-container.component.html',
})
export class BescheidWizardAbschliessenDialogContainerComponent {
  public abschliessen$: Observable<StateResource<CommandResource>>;

  constructor(
    @Inject(DIALOG_DATA) private readonly dialogData: AbschliessenDialogData,
    private readonly dialogRef: DialogRef,
    private readonly bescheidService: BescheidService,
  ) {}

  public onClose(): void {
    this.dialogRef.close();
  }

  public onConfirm(): void {
    this.abschliessen$ = this.bescheidService
      .bescheidErstellungUeberspringen(this.dialogData.vorgangWithEingangResource)
      .pipe(
        tapOnCommandSuccessfullyDone((commandStateResource: StateResource<CommandResource>) =>
          this.dialogRef.close(createDialogResult(commandStateResource)),
        ),
      );
  }
}
