/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { QueryList } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonToggleComponent } from '@ods/system';
import { ButtonToggleGroupComponent } from './button-toggle-group.component';

describe('ButtonToggleGroupComponent', () => {
  let component: ButtonToggleGroupComponent;
  let fixture: ComponentFixture<ButtonToggleGroupComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [ButtonToggleGroupComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(ButtonToggleGroupComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('ngAfterContentInit', () => {
      beforeEach(() => {
        component.passPropsToChild = jest.fn();
      });

      it('should pass props to children', () => {
        const queryList: QueryList<ButtonToggleComponent> = new QueryList();
        queryList.reset([new ButtonToggleComponent(), new ButtonToggleComponent()]);
        component.buttonToggles = queryList;

        component.ngAfterContentInit();

        expect(component.passPropsToChild).toHaveBeenCalledTimes(2);
      });
    });

    describe('passPropsToChild', () => {
      const funcOne: () => number = jest.fn().mockReturnValue(1);
      const funcTwo: () => number = jest.fn().mockReturnValue(2);

      beforeEach(() => {
        component.setFormControlValue = jest.fn().mockReturnValue(funcOne);
        component.isValueSelected = jest.fn().mockReturnValue(funcTwo);
      });

      it('should call setFormControlValue', () => {
        const buttonToggle = new ButtonToggleComponent();
        buttonToggle.value = 'test';

        component.passPropsToChild(buttonToggle);

        expect(component.setFormControlValue).toHaveBeenCalledWith('test');
      });

      it('should set onClick for child', () => {
        const buttonToggle = new ButtonToggleComponent();

        component.passPropsToChild(buttonToggle);

        expect(buttonToggle.onClick).toBe(funcOne);
      });

      it('should call isValueSelected', () => {
        const buttonToggle = new ButtonToggleComponent();

        component.passPropsToChild(buttonToggle);

        expect(component.isValueSelected).toHaveBeenCalled();
      });

      it('should set isSelected for child', () => {
        const buttonToggle = new ButtonToggleComponent();

        component.passPropsToChild(buttonToggle);

        expect(buttonToggle.isSelected).toBe(funcTwo);
      });
    });

    describe('setFormControlValue', () => {
      beforeEach(() => {
        component.fieldControl.setValue = jest.fn();
        fixture.detectChanges();
      });

      it('should return function', () => {
        const result: Function = component.setFormControlValue('test');

        expect(typeof result).toBe('function');
      });

      it('should set value if values are not equal', () => {
        component.setFormControlValue('test')();

        expect(component.fieldControl.setValue).toHaveBeenCalledWith('test');
      });

      it('should not set value if values are equal', () => {
        component.setFormControlValue(null)();

        expect(component.fieldControl.setValue).not.toHaveBeenCalled();
      });
    });

    describe('isValueSelected', () => {
      it('should return function', () => {
        const result: Function = component.isValueSelected();

        expect(typeof result).toBe('function');
      });

      it('should return true', () => {
        const result: boolean = component.isValueSelected()(null);

        expect(result).toBe(true);
      });

      it('should return false', () => {
        const result: boolean = component.isValueSelected()('whatever');

        expect(result).toBe(false);
      });
    });
  });
});
