/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Command, CreateCommand } from '@alfa-client/command-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Store } from '@ngrx/store';
import { createCommand, createCreateCommand } from 'libs/command-shared/test/command';
import { createLoeschAnforderungResource } from 'libs/loesch-anforderung-shared/test/loesch-anforderung';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { Subject } from 'rxjs';
import { LoeschAnforderungLinkRel } from '../loesch-anforderung.linkrel';
import { LoeschAnforderungResource } from '../loesch-anforderung.model';
import { LoeschAnforderungFacade } from './loesch-anforderung.facade';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';
import * as LoeschAnforderungActions from './loesch-anforderung.actions';
import * as LoeschAnforderungSelectors from './loesch-anforderung.selectors';

describe('LoeschAnforderungFacade', () => {
  let facade: LoeschAnforderungFacade;
  let store: Mock<Store>;

  let selectionSubject: Subject<any>;

  beforeEach(() => {
    store = mock(Store);

    selectionSubject = new Subject();

    store.select.mockReturnValue(selectionSubject);
    store.dispatch = jest.fn();

    facade = new LoeschAnforderungFacade(useFromMock(<any>store));
  });

  it('is initialized', () => {
    expect(facade).toBeTruthy();
  });

  describe('LoeschAnfordernCommand', () => {
    it('should select from store', () => {
      facade.getLoeschenAnfordernCommand();

      expect(store.select).toHaveBeenCalledWith(
        LoeschAnforderungSelectors.loeschenAnfordernCommand,
      );
    });
  });

  describe('getLoeschAnforderung', () => {
    it('should select from store', () => {
      facade.getLoeschAnforderung();

      expect(store.select).toHaveBeenCalledWith(LoeschAnforderungSelectors.loeschAnforderung);
    });
  });

  describe('loadLoeschAnforderung', () => {
    const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

    it('should dispatch loadLoeschAnforderung', () => {
      facade.loadLoeschAnforderung(vorgangWithEingang);

      expect(store.dispatch).toHaveBeenCalledWith(
        LoeschAnforderungActions.loadLoeschAnforderung({ vorgangWithEingang }),
      );
    });
  });

  describe('loeschenAnfordern', () => {
    const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
    const command: CreateCommand = createCreateCommand();

    it('should dispatch createCommand action', () => {
      facade.loeschenAnfordern(vorgangWithEingang, command);

      expect(store.dispatch).toHaveBeenCalledWith(
        CommandActions.createCommand({
          resource: vorgangWithEingang,
          linkRel: VorgangWithEingangLinkRel.LOESCHEN_ANFORDERN,
          command,
        }),
      );
    });
  });

  describe('getEndgueltigLoeschenCommand', () => {
    it('should select from store', () => {
      facade.getEndgueltigLoeschenCommand();

      expect(store.select).toHaveBeenCalledWith(LoeschAnforderungSelectors.vorgangLoeschenCommand);
    });
  });

  describe('endgueltigLoeschen', () => {
    const loeschAnforderung: LoeschAnforderungResource = createLoeschAnforderungResource();
    const command: Command = createCommand();

    it('should dispatch action', () => {
      facade.endgueltigLoeschen(loeschAnforderung, command);

      const createCommandAction = CommandActions.createCommand({
        resource: loeschAnforderung,
        linkRel: LoeschAnforderungLinkRel.EXECUTE_LOESCHEN,
        command,
      });
      expect(store.dispatch).toHaveBeenCalledWith(createCommandAction);
    });
  });

  describe('getLoeschAnforderungZuruecknehmenCommand', () => {
    it('should select from store', () => {
      facade.getLoeschAnforderungZuruecknehmenCommand();

      expect(store.select).toHaveBeenCalledWith(
        LoeschAnforderungSelectors.loeschAnforderungZuruecknehmenCommand,
      );
    });
  });

  describe('loeschAnforderungZuruecknehmen', () => {
    it('should dispatch action', () => {
      const loeschAnforderung: LoeschAnforderungResource = createLoeschAnforderungResource();
      const command: Command = createCommand();

      facade.loeschAnforderungZuruecknehmen(loeschAnforderung, command);

      expect(store.dispatch).toHaveBeenCalledWith(
        CommandActions.createCommand({
          resource: loeschAnforderung,
          linkRel: LoeschAnforderungLinkRel.ZURUECKNEHMEN,
          command,
        }),
      );
    });
  });
});
