/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Injectable } from '@angular/core';
import { UntypedFormBuilder, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { isEmpty } from 'lodash-es';
import { AbstractFormService, StateResource } from '@alfa-client/tech-shared';
import { Observable } from 'rxjs';
import { Resource } from '@ngxp/rest';

@Injectable()
export class AktenzeichenEditDialogFormservice extends AbstractFormService {
  static readonly AKTENZEICHEN_FIELD = 'aktenzeichen';

  private vorgang: VorgangWithEingangResource;

  constructor(
    formBuilder: UntypedFormBuilder,
    private readonly vorgangService: VorgangService,
  ) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [AktenzeichenEditDialogFormservice.AKTENZEICHEN_FIELD]: new UntypedFormControl(null),
    });
  }

  protected doSubmit(): Observable<StateResource<Resource>> {
    return this.vorgangService.setAktenzeichen(this.vorgang, this.getValue());
  }

  protected getPathPrefix(): string {
    return AktenzeichenEditDialogFormservice.AKTENZEICHEN_FIELD;
  }

  getAktenzeichenFormControl(): UntypedFormControl {
    return this.getFormControl();
  }

  public getValue(): string {
    const value: string = this.getAktenzeichenFormControl().value;
    return isEmpty(value) ? null : value.trim();
  }

  public setVorgangWithEingangResource(vorgang: VorgangWithEingangResource): void {
    this.vorgang = vorgang;
  }

  public setAktenzeichen(aktenzeichen: string): void {
    this.getFormControl().setValue(aktenzeichen);
  }

  public valueChanges(): Observable<any> {
    return this.getFormControl().valueChanges;
  }

  private getFormControl(): UntypedFormControl {
    return <UntypedFormControl>(
      this.form.controls[AktenzeichenEditDialogFormservice.AKTENZEICHEN_FIELD]
    );
  }
}
