/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ZUSTAENDIGE_STELLE_SERVICE, ZustaendigeStelleService } from '@alfa-client/zustaendige-stelle-shared';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { Component, HostListener, Inject, OnInit } from '@angular/core';
import { Resource } from '@ngxp/rest';
import { InstantSearchResult } from '@ods/system';
import { Observable } from 'rxjs';

//TODO rename to SearchZustaendigeStelleDialogContainerComponent
@Component({
  selector: 'search-zustaendige-stelle-dialog',
  templateUrl: './search-zustaendige-stelle-dialog.component.html',
})
export class SearchZustaendigeStelleDialogComponent<T extends Resource> implements OnInit {
  public searchResults$: Observable<InstantSearchResult<T>[]>;

  constructor(
    private dialogRef: DialogRef,
    @Inject(ZUSTAENDIGE_STELLE_SERVICE)
    private service: ZustaendigeStelleService<T>,
    @Inject(DIALOG_DATA) public title: string,
  ) {}

  ngOnInit(): void {
    this.searchResults$ = this.service.getSearchResultList();
  }

  @HostListener('document:keydown', ['$event'])
  onKeyDownHandler(e: KeyboardEvent) {
    if (e.key === 'Enter' && this.isNotButtonElement(e.target as Element)) {
      e.preventDefault();
    }
  }

  public search(searchBy: string): void {
    this.service.search(searchBy);
  }

  public selectSearchResult(zustaendigeStelle: T): void {
    this.service.selectSearchResult(zustaendigeStelle);
    this.service.clearSearchResult();
    this.dialogRef.close();
  }

  public clearSearchResult(): void {
    this.service.clearSearchResult();
  }

  public closeDialog(): void {
    this.clearSearchResult();
    this.dialogRef.close();
  }

  public isNotButtonElement(e: Element): boolean {
    return e.tagName !== 'BUTTON';
  }
}
