/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.attachment;

import java.io.OutputStream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileModelAssembler;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileService;
import de.ozgcloud.alfa.common.zipdownload.ZipDownloadService;
import de.ozgcloud.alfa.vorgang.VorgangController;
import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(AttachmentByVorgangController.PATH)
@RequiredArgsConstructor
public class AttachmentByVorgangController {
	static final String PATH = "/api/vorgang/{vorgangId}/attachments"; // NOSONAR
	static final String APPLICATION_ZIP_VALUE = "application/zip";

	private final OzgFileService fileService;
	private final ZipDownloadService zipDownloadService;
	private final VorgangController vorgangController;
	private final BinaryFileModelAssembler modelAssembler;

	@GetMapping
	public CollectionModel<EntityModel<OzgFile>> getAllByVorgang(@PathVariable String vorgangId) {
		return modelAssembler.toCollectionModel(fileService.getAttachments(vorgangId));
	}

	@GetMapping(produces = APPLICATION_ZIP_VALUE)
	public ResponseEntity<StreamingResponseBody> download(@PathVariable String vorgangId) {
		return ResponseEntity.ok()
				.header(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=%s", buildZipName(vorgangId)))
				.contentType(MediaType.valueOf(APPLICATION_ZIP_VALUE))
				.body(out -> createZipFile(out, vorgangId));
	}

	String buildZipName(String vorgangId) {
		return vorgangController.getVorgang(vorgangId).getNummer() + "_Anhaenge.zip";
	}

	void createZipFile(OutputStream out, String vorgangId) {
		var ozgFiles = fileService.getAttachments(vorgangId);
		zipDownloadService.write(ozgFiles.toList(), out);
	}
}
