/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.bescheid;

import java.util.Optional;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.bescheid.BescheidServiceGrpc.BescheidServiceBlockingStub;
import de.ozgcloud.bescheid.GrpcBescheid;
import de.ozgcloud.bescheid.GrpcBescheidManagerConfigRequest;
import de.ozgcloud.bescheid.GrpcGetAllBescheidRequest;
import de.ozgcloud.bescheid.GrpcGetBescheidDraftRequest;
import de.ozgcloud.bescheid.GrpcGetBescheidDraftResponse;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class BescheidRemoteService {

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private BescheidServiceBlockingStub bescheidServiceStub;
	@Autowired
	private BescheidMapper bescheidMapper;

	public Optional<Bescheid> getBescheidDraft(String vorgangId) {
		var request = buildGetBescheidDraftRequest(vorgangId);
		var response = bescheidServiceStub.getBescheidDraft(request);
		return getBescheidFromResponse(response).map(bescheid -> bescheidMapper.fromGrpc(bescheid, vorgangId));
	}

	GrpcGetBescheidDraftRequest buildGetBescheidDraftRequest(String vorgangId) {
		return GrpcGetBescheidDraftRequest.newBuilder()
				.setVorgangId(vorgangId)
				.build();
	}

	Optional<GrpcBescheid> getBescheidFromResponse(GrpcGetBescheidDraftResponse response) {
		return response.hasBescheid() ? Optional.of(response.getBescheid()) : Optional.empty();
	}

	public Stream<Bescheid> findByVorgangId(String vorgangId) {
		var request = buildGetAllBescheidRequest(vorgangId);

		var response = bescheidServiceStub.getAll(request);

		return response.getBescheidList().stream().map(grpcBescheid -> bescheidMapper.fromGrpc(grpcBescheid, vorgangId));
	}

	GrpcGetAllBescheidRequest buildGetAllBescheidRequest(String vorgangId) {
		return GrpcGetAllBescheidRequest.newBuilder().setVorgangId(vorgangId).build();
	}

	// TODO Logik in den Service verschieben
	public boolean canCreateBescheidDocument() {
		var response = bescheidServiceStub.getConfig(GrpcBescheidManagerConfigRequest.newBuilder().build());
		return response.hasFeatures() && response.getFeatures().getCanCreateBescheidDocument();
	}
}