/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.hint;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.RootTestFactory;

class HintRootResourceProcessorTest {

	@InjectMocks
	private HintRootResourceProcessor processor;
	@Mock
	private HintService hintService;

	@DisplayName("HintsLink")
	@Nested
	class TestHintsLink {

		@Test
		void shouldCallHintService() {
			processor.process(EntityModel.of(RootTestFactory.create()));

			verify(hintService).hasHints();
		}

		@Test
		void shouldBePresentIfHintsExists() {
			when(hintService.hasHints()).thenReturn(true);

			var model = processor.process(EntityModel.of(RootTestFactory.create()));

			assertThat(model.getLink(HintRootResourceProcessor.HINTS_RELATION)).get().extracting(Link::getHref).isEqualTo(HintController.PATH);
		}

		@Test
		void shouldNotBePresentIfHintsExists() {
			when(hintService.hasHints()).thenReturn(false);

			var model = processor.process(EntityModel.of(RootTestFactory.create()));

			assertThat(model.getLink(HintRootResourceProcessor.HINTS_RELATION)).isNotPresent();
		}
	}
}